@@ -13,6 +13,7 @@
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "insns.h"		/* For MAX_KEYWORD */
 
 static efunc nasm_malloc_error;
 
@@ -83,9 +84,9 @@ void nasm_free (void *q)
 }
 
 #ifdef LOGALLOC
-char *nasm_strdup_log (char *file, int line, char *s)
+char *nasm_strdup_log (char *file, int line, const char *s)
 #else
-char *nasm_strdup (char *s)
+char *nasm_strdup (const char *s)
 #endif
 {
     char *p;
@@ -125,29 +126,33 @@ char *nasm_strndup (char *s, size_t len)
     return p;
 }
 
+#if !defined(stricmp) && !defined(strcasecmp)
 int nasm_stricmp (const char *s1, const char *s2) 
 {
-    while (*s1 && toupper(*s1) == toupper(*s2))
+    while (*s1 && tolower(*s1) == tolower(*s2))
 	s1++, s2++;
     if (!*s1 && !*s2)
 	return 0;
-    else if (toupper(*s1) < toupper(*s2))
+    else if (tolower(*s1) < tolower(*s2))
 	return -1;
     else
 	return 1;
 }
+#endif
 
+#if !defined(strnicmp) && !defined(strncasecmp)
 int nasm_strnicmp (const char *s1, const char *s2, int n) 
 {
-    while (n > 0 && *s1 && toupper(*s1) == toupper(*s2))
+    while (n > 0 && *s1 && tolower(*s1) == tolower(*s2))
 	s1++, s2++, n--;
     if ((!*s1 && !*s2) || n==0)
 	return 0;
-    else if (toupper(*s1) < toupper(*s2))
+    else if (tolower(*s1) < tolower(*s2))
 	return -1;
     else
 	return 1;
 }
+#endif
 
 #define lib_isnumchar(c)   ( isalnum(c) || (c) == '$')
 #define numvalue(c)  ((c)>='a' ? (c)-'a'+10 : (c)>='A' ? (c)-'A'+10 : (c)-'0')
@@ -190,7 +195,7 @@ long readnum (char *str, int *error)
 	radix = 16, r++;
     else if (q[-1]=='H' || q[-1]=='h')
 	radix = 16 , q--;
-    else if (q[-1]=='Q' || q[-1]=='q')
+    else if (q[-1]=='Q' || q[-1]=='q' || q[-1]=='O' || q[-1]=='o')
 	radix = 8 , q--;
     else if (q[-1]=='B' || q[-1]=='b')
 	radix = 2 , q--;
@@ -326,16 +331,18 @@ void standard_extension (char *inname, char *outname, char *extension,
 static struct RAA *real_raa_init (int layers) 
 {
     struct RAA *r;
+    int i;
 
     if (layers == 0) {
 	r = nasm_malloc (LEAFSIZ);
-	memset (r->u.l.data, 0, sizeof(r->u.l.data));
 	r->layers = 0;
+	memset (r->u.l.data, 0, sizeof(r->u.l.data));
 	r->stepsize = 1L;
     } else {
 	r = nasm_malloc (BRANCHSIZ);
-	memset (r->u.b.data, 0, sizeof(r->u.b.data));
 	r->layers = layers;
+	for ( i = 0 ; i < RAA_LAYERSIZE ; i++ )
+	  r->u.b.data[i] = NULL;
 	r->stepsize = RAA_BLKSIZE;
 	while (--layers)
 	    r->stepsize *= RAA_LAYERSIZE;
@@ -362,15 +369,15 @@ void raa_free (struct RAA *r)
 
 long raa_read (struct RAA *r, long posn) 
 {
-    if (posn > r->stepsize * LAYERSIZ(r))
-	return 0L;
+    if (posn >= r->stepsize * LAYERSIZ(r))
+	return 0;		/* Return 0 for undefined entries */
     while (r->layers > 0) {
 	ldiv_t l;
 	l = ldiv (posn, r->stepsize);
 	r = r->u.b.data[l.quot];
 	posn = l.rem;
-	if (!r)			       /* better check this */
-	    return 0L;
+	if (!r)
+	    return 0;		/* Return 0 for undefined entries */
     }
     return r->u.l.data[posn];
 }
@@ -382,16 +389,18 @@ struct RAA *raa_write (struct RAA *r, long posn, long value)
     if (posn < 0)
 	nasm_malloc_error (ERR_PANIC, "negative position in raa_write");
 
-    while (r->stepsize * LAYERSIZ(r) < posn) {
+    while (r->stepsize * LAYERSIZ(r) <= posn) {
 	/*
-	 * Must go up a layer.
+	 * Must add a layer.
 	 */
 	struct RAA *s;
+	int i;
 
 	s = nasm_malloc (BRANCHSIZ);
-	memset (s->u.b.data, 0, sizeof(r->u.b.data));
+	for ( i = 0 ; i < RAA_LAYERSIZE ; i++ )
+	    s->u.b.data[i] = NULL;
 	s->layers = r->layers + 1;
-	s->stepsize = RAA_LAYERSIZE * r->stepsize;
+	s->stepsize = LAYERSIZ(r) * r->stepsize;
 	s->u.b.data[0] = r;
 	r = s;
     }
@@ -465,9 +474,9 @@ void *saa_wstruct (struct SAA *s)
     return p;
 }
 
-void saa_wbytes (struct SAA *s, void *data, long len) 
+void saa_wbytes (struct SAA *s, const void *data, long len) 
 {
-    char *d = data;
+    const char *d = data;
 
     while (len > 0) {
 	long l = s->end->length - s->end->posn;
@@ -636,11 +645,11 @@ void saa_fpwrite (struct SAA *s, FILE *fp)
  * by the scanner.
  */
 #include "names.c"
-static char *special_names[] = {
+static const char *special_names[] = {
     "byte", "dword", "far", "long", "near", "nosplit", "qword",
-    "short", "to", "tword", "word"
+    "short", "strict", "to", "tword", "word"
 };
-static char *prefix_names[] = {
+static const char *prefix_names[] = {
     "a16", "a32", "lock", "o16", "o32", "rep", "repe", "repne",
     "repnz", "repz", "times"
 };
@@ -753,9 +762,9 @@ int stdscan (void *private_data, struct tokenval *tv)
 	if ((tv->t_integer=bsi(ourcopy, special_names,
 				  elements(special_names)))>=0)
 	    return tv->t_type = TOKEN_SPECIAL;
-	if (!strcmp(ourcopy, "seg"))
+	if (!nasm_stricmp(ourcopy, "seg"))
 	    return tv->t_type = TOKEN_SEG;
-	if (!strcmp(ourcopy, "wrt"))
+	if (!nasm_stricmp(ourcopy, "wrt"))
 	    return tv->t_type = TOKEN_WRT;
 	return tv->t_type = TOKEN_ID;
     } else if (*stdscan_bufptr == '$' && !isnumchar(stdscan_bufptr[1])) {
@@ -1000,7 +1009,7 @@ long reloc_wrt (expr *vect)
 /*
  * Binary search.
  */
-int bsi (char *string, char **array, int size) 
+int bsi (char *string, const char **array, int size) 
 {
     int i = -1, j = size;	       /* always, i < index < j */
     while (j-i >= 2) {
@@ -1084,19 +1093,24 @@ char *nasm_strcat(char *one, char *two)
     return rslt;
 }
 
-void null_debug_routine()
-{
-}
+void null_debug_init(struct ofmt *of, void *id, FILE *fp, efunc error ) {}
+void null_debug_linenum(const char *filename, long linenumber, long segto) {}
+void null_debug_deflabel(char *name, long segment, long offset, int is_global, char *special) {}
+void null_debug_routine(const char *directive, const char *params) {}
+void null_debug_typevalue(long type) {}
+void null_debug_output(int type, void *param) {}
+void null_debug_cleanup(void){}
+
 struct dfmt null_debug_form = {
     "Null debug format",
     "null",
+    null_debug_init,
+    null_debug_linenum,
+    null_debug_deflabel,
     null_debug_routine,
-    null_debug_routine,
-    null_debug_routine,
-    null_debug_routine,
-    null_debug_routine,
-    null_debug_routine,
-    null_debug_routine,
+    null_debug_typevalue,
+    null_debug_output,
+    null_debug_cleanup
 };
 
 struct dfmt *null_debug_arr[2] = { &null_debug_form, NULL };
